@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 # Copyright (c) 2016 Mikkel Schubert <MikkelSch@gmail.com>
 #
@@ -19,8 +19,6 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-import paleomix.resources
-
 import paleomix.common.versions as versions
 
 from paleomix.resources import rscript
@@ -31,10 +29,12 @@ def requirement(module, checks=versions.Any(), cache={}):
     result = cache.get(key)
     if result is None:
         filename = rscript("common", "requires.r")
-        result = versions.Requirement(call=("Rscript", filename, module),
-                                      search="d0fd3ea6: (\d+)\.(\d+)(?:\.(\d+))?",
-                                      checks=checks,
-                                      name="R module: {}".format(module))
+        result = versions.Requirement(
+            call=("Rscript", filename, module),
+            search=r"d0fd3ea6: (\d+)\.(\d+)(?:\.(\d+))?",
+            checks=checks,
+            name="R module: {}".format(module),
+        )
 
         cache[(module, checks)] = result
 
